"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var config_1 = require("../config");
var base_1 = __importDefault(require("./base"));
var FetchInterceptor = /** @class */ (function (_super) {
    __extends(FetchInterceptor, _super);
    function FetchInterceptor(mocker) {
        var _this = _super.call(this, mocker) || this;
        if (FetchInterceptor.instance) {
            return FetchInterceptor.instance;
        }
        FetchInterceptor.instance = _this;
        _this.fetch = _this.global.fetch.bind(_this.global);
        _this.intercept();
        return _this;
    }
    FetchInterceptor.setupForUnitTest = function (mocker) {
        var global = base_1.default.getGlobal();
        if (!global.fetch) {
            global.fetch = function () { };
        }
        return new FetchInterceptor(mocker);
    };
    /**
     * https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API/Using_Fetch
     * Intercept fetch object.
     */
    FetchInterceptor.prototype.intercept = function () {
        var me = this;
        this.global.fetch = function () {
            var args = __spreadArray([], arguments);
            var url;
            var params;
            // https://developer.mozilla.org/en-US/docs/Web/API/Request
            // Note: the first argument of fetch maybe a Request object.
            if (typeof args[0] === 'object') {
                url = args[0].url || '';
                params = args[0];
            }
            else {
                url = args[0];
                params = args[1];
            }
            var method = params && params.method ? params.method : 'GET';
            return new Promise(function (resolve, reject) {
                var match = me.matchMockRequest(url, method);
                if (match) {
                    var requestInfo = __assign({ url: url }, params);
                    requestInfo.query = me.getQuery(url);
                    me.doMockRequest(match, requestInfo, resolve);
                }
                else {
                    me.fetch.apply(me, args).then(resolve).catch(reject);
                }
            });
        };
        return this;
    };
    /**
     * Make mock request.
     * @param {MockItemInfo} match
     * @param {FetchRequestInfo} requestInfo
     * @param {Function} resolve
     */
    FetchInterceptor.prototype.doMockRequest = function (match, requestInfo, resolve) {
        var _this = this;
        if (match.file) {
            Promise.resolve().then(function () { return __importStar(require(process.env.HRM_MOCK_DIR + "/" + match.file)); }).then(function (mock) {
                var mockResponse = _this.getMockResponse(mock.default, match, requestInfo);
                _this.doMockResponse(mockResponse, match, resolve);
            });
            return;
        }
        var mockResponse = this.getMockResponse(match.response, match, requestInfo);
        this.doMockResponse(mockResponse, match, resolve);
    };
    /**
     * Make mock request.
     * @param {any} response
     * @param {FetchRequestInfo} requestInfo
     * @param {Function} resolve
     */
    FetchInterceptor.prototype.doMockResponse = function (response, match, resolve) {
        if (match) {
            if (match.delay && match.delay > 0) {
                setTimeout(function () {
                    resolve(response);
                }, +match.delay);
            }
            else {
                resolve(response);
            }
            return;
        }
    };
    /**
     * https://developer.mozilla.org/en-US/docs/Web/API/Response
     * Format mock data.
     * @param {any} mockResponseConfig
     * @param {MockItemInfo} match
     * @param {FetchRequestInfo} requestInfo
     */
    FetchInterceptor.prototype.getMockResponse = function (mockResponseConfig, match, requestInfo) {
        var data = typeof mockResponseConfig === 'function' ? mockResponseConfig(requestInfo) : mockResponseConfig;
        var status = match.status || 200;
        var statusText = config_1.HTTPStatusCodes[status] || '';
        var headers = typeof Headers === 'function'
            ? new Headers(__assign(__assign({}, match.header), { 'is-mock': 'yes' }))
            : __assign(__assign({}, match.header), { 'is-mock': 'yes' });
        var body = typeof Blob === 'function'
            ? new Blob([typeof data === 'string' ? data : JSON.stringify(data)])
            : data;
        if (typeof Response === 'function') {
            var response_1 = new Response(body, { status: status, statusText: statusText, headers: headers });
            Object.defineProperty(response_1, 'url', { value: requestInfo.url });
            return response_1;
        }
        var response = {
            // If you should need some other complex object, you could define it in your mock file.
            body: body,
            bodyUsed: false,
            headers: headers,
            ok: true,
            redirected: false,
            status: status,
            statusText: statusText,
            url: requestInfo.url,
            type: 'basic',
            // response data depends on prepared data
            json: function () { return Promise.resolve(data); },
            arrayBuffer: function () { return Promise.resolve(data); },
            blob: function () { return Promise.resolve(body); },
            formData: function () { return Promise.resolve(data); },
            text: function () { return Promise.resolve(typeof data === 'string' ? data : JSON.stringify(data)); },
            // other methods that may be used
            clone: function () { return response; },
            error: function () { return response; },
            redirect: function () { return response; },
        };
        return response;
    };
    return FetchInterceptor;
}(base_1.default));
exports.default = FetchInterceptor;
//# sourceMappingURL=fetch.js.map